module RailsIdle
  module Storage
    class RailsCache < Base
      
      def add(path, execution_time = nil)
        super
        if execution_time
          increment(path, COUNTER_KEY)
          increment(path, EXECUTION_KEY, (execution_time * 1000).round)
        end
      end
      
      def get(path, key)
        begin
          Rails.cache.read build_name(path, key)
        rescue Exception => e # Redis workaround
          (Rails.cache.read build_name(path, key), raw: true).to_i
        end
      end
      
      def reset(path)
        [COUNTER_KEY, EXECUTION_KEY].each do |key|
           Rails.cache.delete build_name(path, key)
        end
      end
      
      private
      
      def increment(path, key, amount = 1)
        Rails.cache.increment build_name(path, key), amount
      end
      
      def build_name(path, key)
        "#{RailsIdle}.#{path}.#{key}"
      end
      
    end
  end
end